<?php

/****************************************************************************

  Copyright (C) 2017-2020  Sendatel S.A.

  This file is part of TUIDD DRUPAL SDK.
  TUIDD DRUPAL SDK is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  TUIDD DRUPAL SDK is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TUIDD DRUPAL SDK.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

require_once('TuiddException.php');

class TuiddClient
{
	private $m_ClientSecret = '';
	private $m_ClientId = '';
	private $m_UrlRedirect = '';
	private $m_UrlDiscovery = '';
	private $m_UrlTuiddContent = '';
	private $m_UrlProveedoraToken = '';
	private $m_UrlUserInfoRedirect = '';
	
	private $m_AuthCode = '';
	private $m_accessTokenData = '';
	private $m_UserInfo = '';
	private $m_PaymentToken = '';

	private $m_LastServiceResult = '';
	private $m_UrlSendService = '';

	private $m_ContainerDiv = '';
	private $m_tuiddSDKPath = '';
	private $m_Scope = '';
	private $m_State= '';
	private $m_Nonce= '';
    private $m_StateNonceCallback= '';
	private $m_Security = '';
	private $m_SaveUserInfo	= '';
	private $m_ExecuteCallback = '';

	private $m_Language = '';
	private $m_HomePageUrl = '';

	private $m_init_time = '';
    private $__DEBUG__ = '';

	function __construct()
	{
		$internal_conf = parse_ini_file( "internal_params.conf" );
		$this->m_UrlDiscovery = $internal_conf['tuidd_discovery'];
		$this->m_UrlTuiddContent = $internal_conf['tuidd_sdk'];

		$this->m_Scope = variable_get('scope', '');
		$this->m_ClientId = variable_get('client_id', '');
		$this->m_ClientSecret = variable_get('client_secret', '');
		$this->m_HomePageUrl = variable_get('home_page_url', '');
		$this->m_UrlUserInfoRedirect = variable_get('result_login_uri', '');		
		$this->m_Security = variable_get('acr_values', '');
		$this->m_PaymentToken = variable_get('payment_token', '');
		$this->m_UrlRedirect = variable_get('redirect_uri', '');
		$this->m_SaveUserInfo = variable_get('save_user_info', '');
		$this->m_ExecuteCallback = variable_get('execute_callback', '');
		$this->m_StateNonceCallback = variable_get('state_nonce_callback_function', '');
		$this->m_ContainerDiv = variable_get('container_div', '');
        $this->__DEBUG__ = variable_get('DEBUG', 'false');
	}


	/**
	 * Gets Singleton's instance of TuiddClient 
	 * @param  none
	 * @return [type]              [description]
	 */
	public static function getSingleton()
	{
		if ( !isset($_SESSION['tuidd']) || empty($_SESSION['tuidd']) ) {
			return new self();
		}
		else
    	{
            if( $_SESSION['tuidd'] instanceof TuiddClient ) {
                return $_SESSION['tuidd'];
            } else /* if( $_SESSION['tuidd'] instanceof string ) */{
                $ret = new self();
                $ret = unserialize($_SESSION['tuidd']);
                return $ret;
            } 
    	}
	}


    /**
     * serializing the tuidd session
     */
	private function updateContent()
    {
		$_SESSION['tuidd'] = serialize($this);
	}


	/**
     * LOGOUT and destroy session
     * @param  none
     * @return Bool True or Redirect
     */
    public function logout()
    {
        $home = $this->m_HomePageUrl;
        session_destroy('sdk-session');
        $_SESSION['tuidd'] = '';
        header("Location:" . $home );
    }


	/**
	 * Request fundamental Tuidd urls
	 * @return String Tuidd URI for making requests 
	 */
	public function discovery()
	{
		$mcg_url = $this->m_UrlDiscovery;

		$headers = array();
        $headers[]     = 'Authorization: Basic '.base64_encode( $this->m_ClientId . ':' . $this->m_ClientSecret );
        $headers[]     = 'Content-Type: application/x-www-form-urlencoded';

		$arrParams = array( 'redirect_uri' => $this->m_UrlRedirect );
		$params = http_build_query($arrParams);

		$result = $this->sendCurl($mcg_url, $params, $headers);
		$response_json = json_decode($result['response'], true);

		$urls_json = $response_json['response']['apis']['operatorid']['link'];
        if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". basename(__FILE__, ".php") ." :: DISCOVERY URLS :: " . $urls_json);
        }

		$request_url = $urls_json[0]['href'];
		$this->m_UrlProveedoraToken = $urls_json[1]['href'];
		$this->m_UrlUserInfo = $urls_json[2]['href'];
		$this->m_UrlSendService = $urls_json[3]['href'];

		$this->updateContent();

		return $request_url;
	}


	/**
	 * If Discovery URLS had been set
	 * @return Bool
	 */
	public function hasDiscoveryUrls() {
		return ( isset($this->m_UrlProveedoraToken) && $this->m_UrlProveedoraToken != '');
	}


	//public bool getToken (TuiddGetTokenReq $object )
	public function getToken( $code )
	{
        if ($this->m_UrlProveedoraToken == '')
        {
            syslog(LOG_WARNING, "TUIDD :: TOKEN ::TOKEN URL REQUEST :: VACIO"); 
            $this->discovery();
        }

		if ($this->m_AuthCode == '' && $this->m_UrlProveedoraToken != '')
		{
			$this->m_AuthCode = $code;
			$url = $this->m_UrlProveedoraToken;

			$headers = array();
            $headers[]     = 'Authorization: Basic '.base64_encode( $this->m_ClientId . ':' . $this->m_ClientSecret );
            $headers[]     = 'Content-Type: Application/x-www-form-urlencoded';

			$arrParams = array('grant_type' => 'authorization_code', 'code' => $this->m_AuthCode, 'redirect_uri' => $this->m_UrlRedirect );
			$params = http_build_query($arrParams);
			
			$result = $this->sendCurl($url, $params, $headers);
			$aux = (array)json_decode($result['response']);

			if ( empty($aux) || ! empty($aux['error']) )
			{
				if ( empty($aux)) 
				{
					throw new TuiddException("Error Durante el pedido del accessToken. Respuesta vacia.");
				}
				else
				{
					throw new TuiddException("Error Durante el pedido del accessToken. La descripcion del error es la siguiente: " . $aux['error_descripcion'] );
				}
			}
			else
			{
				$this->m_accessTokenData = $aux;
				$this->m_init_time = time();
				//$_SESSION['tuidd'] = serialize( $this );
				return true;
			}
		}				
	}


	// public bool getUserInfo (TuiddGetUserInfoReq $object )
	public function requestUserInfo()
	{
        if ($this->m_UrlProveedoraToken == '')
        {
            syslog(LOG_WARNING, "TUIDD :: USERINFO :: TOKEN URL REQUET :: VACIO"); 
        }

		if ($this->m_UserInfo == '' && $this->m_UrlProveedoraToken != '')
		{
			$url = $this->m_UrlProveedoraToken;		

			$headers = array('Content-Type: application/x-www-form-urlencoded', 'Authorization: Bearer ' . $this->m_accessTokenData['access_token']);
			$arrParams = array('grant_type' => 'authorization_code', 'code' => $this->m_AuthCode, 'redirect_uri' => $this->m_UrlRedirect);

			$params = http_build_query($arrParams);
			$result = $this->sendCurl($url, $params, $headers);
			$aux = (array)json_decode($result['response']);

			if ( empty($aux) || ! empty($aux['error_description']) )
			{
				if ( empty($aux)) 
				{
					throw new TuiddException("Error Durante el pedido del accessToken. respuesta vacio");
				}
				else
				{
					throw new TuiddException("Error Durante el pedido del UserInfo. La descrición del error es el siguiente: " . $aux['error_description'] );
				}
			}
			else {
				$this->m_UserInfo = $aux;
				return true;
			}
		}
	}


	/** 
     * Esta función sirve para procesar los datos del usuario. 
     * Que hacer con esos datos depende del poveedor.
     * En este caso se realiza un redirect al servicio. 
     */
	public function cfReceive() 
	{
		$user_info = $this->m_UserInfo;

        if ( $user_info != '')
        {
            $user_info['client_info']  = base64_encode( $this->m_ClientId . ':' . $this->m_ClientSecret );
            $user_info['access_token']  = $this->m_accessTokenData['access_token'];
            $user_info['id_token']  = $this->m_accessTokenData['id_token'];

            $this->m_UserInfo = $user_info;
            $this->updateContent();

            // Si quiere crear usuario 
            // Si quiere ejecutar el callback
            // Redirifir a user
            if ($this->m_SaveUserInfo)
            {
                $this->tuidd_create_and_login_user();	
            }
            if ($this->m_ExecuteCallback)
            {
                require_once(drupal_get_path('module', 'tuidd') . '/libs/tuidd_lib/Tuidd_callback_function_file.inc');
                call_user_func( 'tuidd_callback_function', '');
            }
            drupal_goto($this->m_UrlUserInfoRedirect);
        }
	}


	/**
     * Setting Nonce.
     * @param  string nonce
     */
    public function setNonce ( $nonce )
    {
        $this->m_Nonce = $nonce;
        $this->updateContent();
    }


    /**
     * Setting State.
     * @param  string state
     */
    public function setState ( $state )
    {
        $this->m_State = $state;
        $this->updateContent();
    }


    /**
     * Callback Function for consuming a service.
     * @param  array Service information to be consumed
     * @return array Status
     */
    public function consumeService ( $service )
    {
        $headers = array();
        $headers[]     = 'Authorization: Bearer '.base64_encode( $this->m_ClientId . ':' . $this->m_ClientSecret );
        $headers[]     = 'Content-Type: Application/json';

        $url = $this->m_UrlSendService;

        // Para Array multidimensionales
        $params = json_encode($service, JSON_FORCE_OBJECT);

        $result = $this->sendCurl($url, $params, $headers);

        $r = $result['response'];
        if (strpos($r, "{" ) > 1)
        {
            $r = substr($r, strpos($r, "{" )-1, strlen($r));
        }
        $response = json_decode( $r, true);

        $st = 1;
        $statusMessage;

        if ($result['status'] == 200)
        {
            if ($response['transaction_id'] && $response['seq_num'] )
            {
                $rta = array( 'transaction_id' => $response['transaction_id'],
                              'seq_num' => $response['seq_num'],
                );
                $st = 0;
                if ($response['url_redirect'])
                {
                    $rta['url_redirect'] = $response['url_redirect'];
                }
            }
            else
            {
                $rta = null; //$response['error_data'];
                $statusMessage = 'Error. Respuesta Incompleta. Falta ';

                if (! $response['transaction_id'])
                {
                    $statusMessage = $statusMessage . 'el parametro transaction_id.';
                }
                else if (! $response['seq_num'])
                {
                    $statusMessage = $statusMessage . 'el parametro seq_num.';
                }
            }
        }
        else if ($result['status'] == 400)
        {
            $statusMessage = 'BAD REQUEST';
        }
        else if ($result['status'] == 500)
        {
            $statusMessage = 'INTERNAL SERVER ERROR';
            if ($response['status']['error_data']['error_description']) {
                $statusMessage = "Error: " . $response['status']['error_data']['error'] ." - ". $response['status']['error_data']['error_description'];
            }
        }
        else if ($result['status'] == 401)
        {
            $statusMessage = 'UNAUTHORIZED';
        }

        $serviceResponse = array(
            'result' => $st,
            'status' => array(
                'description' => $statusMessage,
                'code' => $result['status']
            ),
            'response' => $rta,
        );

        return $serviceResponse;
    }


    /**
     * Get Access TOKEN.
     * @param  array Service information to be consumed
     * @return array Status
     */
    public function getAccessToken()
    {
        return $this->m_accessTokenData['access_token'];
    }


    /**
     * Get Survey service TOKEN.
     * @param  array Service information to be consumed
     * @return array Status
     */
    public function getSurveyServiceToken()
    {
        return $this->m_accessTokenData['tuidd.com.survey_token'];
    }


    /**
     * Get Advertisement service TOKEN.
     * @param  array Service information to be consumed
     * @return array Status
     */
    public function getAdvertisingServiceToken()
    {
        return $this->m_accessTokenData['tuidd.com.advertising_token'];
    }


    /**
     * Get Notification service TOKEN.
     * @param  array Service information to be consumed
     * @return array Status
     */
    public function getNotificationServiceToken()
    {
        return $this->m_accessTokenData['tuidd.com.notification_token'];
    }


    /**
     * Config param getter
     * @return client_id
     */
    public function getClientID()
    {
        return $this->m_ClientId;
    }


    /**
     * Config param getter
     * @return scope
     */
    public function getScope()
    {
        return $this->m_Scope;
    }


	/**
	 * Config param getter
	 * @return security_level
	 */
	public function getSecurityLevel()
    {
		return $this->m_Security;
	}


	/**
	 * Config param getter
	 * @return redirect_uri
	 */
	public function getRedirectURI()
    {
		return $this->m_UrlRedirect;
	}


	/**
	 * Config param getter
	 * @return default_language
	 */
	public function getLanguage()
    {
		return $this->m_Language;
	}
	

	/**
	 * Config param getter
	 * @return container_div
	 */
	public function getContainerDiv()
    {
		return $this->m_ContainerDiv;
	}
	

	/**
	 * Config param getter
	 * @return tuidd_sdk_path
	 */
	public function getTuiddSDKPath()
    {
		return $this->m_tuiddSDKPath;
	}


	/**
	 * Config param getter
	 * @return home_page_url
	 */
	public function getHomePageURL()
    {
		return $this->m_HomePageUrl;
	}


	/**
	 * Config param getter
	 * @return home_page_url
	 */
	public function getDiscoveryUrl()
    {
		return $this->m_UrlDiscovery;
	}


	/**
	 * Config param getter
	 * @return url_tuidd_content
	 */
	public function getTuiddContentURL()
    {
		return $this->m_UrlTuiddContent;
	}


	/**
	 * Simply packs User information and delivers it 
	 * @return array  userInfo
	 */
	public function getUserInfo()
    {
		$userInfo = $this->m_UserInfo;
		return $userInfo;
	}


	/**
	 * Get Notification service TOKEN. 
	 * @param  array Service information to be consumed
	 * @return array Status
	 */	
	public function getStateNonceCallback()
	{
		return $this->m_StateNonceCallback;
	}


    public function getState()
    {
        return $this->m_State;
    }


	/**
	 * Genera un state que contiene el state del sitio y un state interno del
	 * sdk.
	 * 
	 * @param  string $clientState el state que notifico el sitio.
	 * @param  string $sdkState    state con informaicon util para el sdk.
	 * @return string              json con los states en base64_url.
	 */
	public function generateSdkState($clientState, $sdkState='')
	{
		$jsonState = json_encode([
			'client_state' => $clientState,
			'sdk_state' => $sdkState
        ]);
		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". __METHOD__ ." :: state: $jsonState");
		}

		return self::base64urlEncode($jsonState);
	}


	/**
	 * deserializa el rawState devolciendo un array asociativo con los campos:
	 * [ 'client_state' => $clientState, 'sdk_state' => $sdkState ]
	 * 
	 * @param  String $rawState json con los states en base64_url.
	 * @return array            Array con 2 campos client_state y action_state.
	 */
	public function readSdkState($rawState)
	{
		$jsonState = self::base64urlDecode($rawState);

		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". __METHOD__ ." :: state: $jsonState");
		}
		$ret = json_decode($jsonState, TRUE);
		if ($this->__DEBUG__)
		{
			syslog(LOG_INFO, "TUIDD SDK :: ". __METHOD__ ." :: return: ".print_r($ret,TRUE));
		}

		return $ret;
	}


    public function getNonce()
    {
        return $this->m_Nonce;
    }


	/**
	 * Get Survey service TOKEN. 
	 * @param  array 			Service information	to be consumed
	 * @return array            Status
	 */
	public function generateStateAndNonce()
    {
        syslog(LOG_INFO, "". basename(__FILE__, ".php") ." :: callback state func :: " . $this->getStateNonceCallback());
        $statenoncecf = 'stateNonceCallbackFunction';
        $statenoncecf = $this->getStateNonceCallback();

        if ( $statenoncecf != '' )
        {
            require_once( drupal_get_path('module', 'tuidd') . '/libs/tuidd_lib/Tuidd_callback_function_file.inc' );
            call_user_func( $statenoncecf );
        }
	}


	/**
	 * consulta si esta habilitado el debug
	 * @return boolean isDebugging
	 */	
	public function isDebugging()
	{
		return $this->__DEBUG__;
	}


	/**
	 * validateLogin Validates id_token
	 * @return true  if valid
	 */
	public function validateLogin() 
	{
		require_once( 'lib/jwtManager/JWT.php' ); 

		$id_token = $this->m_accessTokenData['id_token'];
		$cs = $this->m_ClientSecret;
		
		try 
		{
			$decoded = JWT::decode($id_token, $cs, array('HS256'));
			$decoded = (array) $decoded;
			return true;
		} 
		catch(Exception $e) 
		{
			echo 'Message: ' .$e->getMessage();
		}		
	}


	/**
     * Esta funcion es la que esta encargada de hacer los envios por curl y transmitir la informacion
     */
	private	function sendCurl($url,$params, $headers)
	{
		
		$params = urldecode($params);
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HTTPGET, FALSE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
               
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER,  $headers);

        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

        $response = curl_exec($ch);

        $header_size     = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $responseBody    = substr($response, $header_size);
        $http_status     = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if ( curl_error($ch) )
        {
        	throw new Exception("Error Durante el envio de Curl");
        }
        else {curl_close($ch);}

		return array(
            'status' => $http_status,
            'response' => $responseBody
        );
	}	


    private function curl_file_get_contents($file)
    {
        if (function_exists('curl_version'))
        {
            $curl = curl_init();
            curl_setopt($curl, CURLOPT_URL, $file);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);

            $content = curl_exec($curl);
            curl_close($curl);
        }
        else if (file_get_contents(__FILE__) && ini_get('allow_url_fopen'))
        {
            $arrContextOptions=array(
                "ssl"=>array(
                    "verify_peer"=>false,
                    "verify_peer_name"=>false,
                ),
            ); 
            $content = file_get_contents($file, false, stream_context_create($arrContextOptions));
        }
        else
        {
            echo 'You have neither cUrl installed nor allow_url_fopen activated. Please setup one of those!';
        }
        return $content;
    }


	public static function base64urlEncode($str)
	{
		return rtrim( strtr( base64_encode( $str ), '+/', '-_'), '=');
	}


	public static function base64urlDecode($str)
	{
		return base64_decode( strtr( $str, '-_', '+/') . str_repeat('=', 3 - ( 3 + strlen( $str )) % 4 ));
	}


    private function printStatus($texto,$valor)
    {
    	echo "--> $texto";
		echo '<pre>';print_r($valor);echo  '</pre>';
    }


    private function name($length = 8, $unique = FALSE)
    {
        $values = array_merge(range(65, 90), range(97, 122), range(48, 57));
        $max = count($values) - 1;
        $counter = 0;

        do {
            // if ($counter == static::MAXIMUM_TRIES) {
            //   throw new \RuntimeException('Unable to generate a unique random name');
            // }
            $str = chr(mt_rand(97, 122));
            for ($i = 1; $i < $length; $i++) {
                $str .= chr($values[mt_rand(0, $max)]);
            }
            $counter++;
        } while ($unique && isset($this->names[$str]));

        if ($unique) {
            $this->names[$str] = TRUE;
        }
        return $str;
	}


    private  function load_user_by_phone( $msisdn ) 
	{
        $query = 'SELECT usr.uid
	    FROM {users} AS usr
	    WHERE usr.phone = :msisdn
	    LIMIT 1';

        $result = db_query($query, array(':msisdn' => $msisdn));
        $user_id = $result->fetchCol();

        return user_load_multiple($user_id);
	}


	private function load_user_by_pcr( $pcr ) 
	{
	  
        $query = 'SELECT usr.uid
	    FROM {users} AS usr
	    WHERE usr.pcr = :pcr
	    LIMIT 1';

        $result = db_query($query, array(':pcr' => $pcr));
        $user_id = $result->fetchCol();

        return $user_id; 
	}


	private function tuidd_create_and_login_user( )
	{
		global $user;
		
		$user_id = $this->load_user_by_pcr( $this->m_UserInfo['sub'] );
		if ( empty($user_id) ) 
		{
			$user = array(
                'name' => $this->m_UserInfo['name'],
                'pass' =>  user_password(),
                'mail' => $this->m_UserInfo['email'],
                'status' => 1,
                'init' => '',# $init,
                'pcr' => $this->m_UserInfo['sub'],
                'msisdn' => $this->m_UserInfo['msisdn'],
                'roles' => array(
                    DRUPAL_AUTHENTICATED_RID => 'authenticated user'
                ),
			);
			// The first parameter is sent blank so a new user is created.
			$user = user_save('', $user);
		}
		else
		{
			$user = user_load($user_id);
		}
	  	
	  	$form_state = array();
	   	$form_state['uid'] = $user_id;
	   	user_login_finalize($form_state);
	}

}

?>
