<?php

/****************************************************************************

	tuidd's PHP SDK version 2.0.0

	Copyright (C) 2017-2020  Sendatel S.A.

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

****************************************************************************/

require_once('TuiddException.php');
require_once('TuiddClient.php');

class TuiddServicesClient
{
	/**
	 * Gets Singleton's instance of TuiddClient 
	 * @param  none
	 * @return [type] [description]
	 */
	public static function getSingleton()
	{
		if ( !isset($_SESSION['tuiddsc']) || empty($_SESSION['tuiddsc']) ) {
			return new self();
		}
		else
    	{
		if( $_SESSION['tuiddsc'] instanceof TuiddServicesClient ) {
                    return $_SESSION['tuiddsc'];
            } else {
                    $ret = new self();
                    $ret = unserialize($_SESSION['tuiddsc']);
                    return $ret;
            } 
    	}
	}


	/**
	 * Envía una publicidad con la información recibida en los parámetros. 
	 * 
	 * @param  string $title [titulo de la publicidad]
	 * @param  string $message [mensaje de la publicidad]
	 * @param  string $link [link de la publicidad]
	 * @param  string $image_link [link de la imagen publicidad]
	 * @param  string $redirect_link [link de la redirect luego de recibir publicidad]
	 * @param  array $authInfo [description]
	 * @param  array $extrParams [description]
	 * @return array
	 *  int $status Estado de envío de publicidad
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public function sendAdvertisement($service_id, $authInfo, $title=NULL, $message=NULL, $link=NULL, $image_link=NULL, $redirect_link=NULL, $source=null, $extraParams=NULL )
	{
		TuiddClient::getSingleton()->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();
        $serviceInfo = array( );

        if ( !$authInfo ) {
            return json_encode(array('error' => 'Parametro AuthInfo vacio'));
        }

        if ( $service_id ) {
            $serviceInfo['service_id'] = $service_id; 

            if ( $title )
                {
                    $serviceInfo['title'] = $title;
                }
            if ( $message )
                {
                    $serviceInfo['message'] = $message;
                }
            if ( $link )
                {
                    $serviceInfo['link'] = $link;
                }
            if ( $image_link )
                {
                    $serviceInfo['image_link'] = $image_link;	
                }	

            if ($redirect_link)
                {
                    $serviceInfo['redirect_link'] = $redirect_link;
                }
        }
        /* else { */
        /*     return json_encode(array('error' => 'Service ID vacio')); */
        /* } */

 		if ($source)
        {
            if ($service_id) {
                if ($source != 'service_id')
                    return json_encode(array('error' => 'El campo source debe ser "service_id" cuando se envia Service ID'));
                $serviceInfo['source'] = 'service_id';
            }
            else {
                $serviceInfo['source'] = $source;
            }
        }
        else {
            $serviceInfo['source'] = 'any';
        }

		$display = 'url_redirect';
		if ($extraParams['display']) 
		{
			$display = $extraParams['display'];
		}
		
		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(1,20),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'advertising',
			'display' => $display,
			'service_info' => $serviceInfo
		);

		$result = $tuidd->consumeService( $service );
		
		return $result;
	}


    /**
	 * Consume un servicio de pago . 
	 * 
	 * @param  [type] $price [titulo de la publicidad]
	 * @param  [type] $detail [description]
	 * @param  [type] $concept [description]
	 * @param  [type] $currency [description]
	 * @param  [type] $authInfo [description]
	 * @param  [type] $extraParams [description]
	 * @return array
	 *  int $status Estado de envío de publicidad
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public static function sendPayment ( $price, $detail, $concept, $currency, $authInfo, $extraParams=NULL )
	{
		$tuidd = TuiddClient::getSingleton();
        $tuidd->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();

        if ( $price && $detail && $concept && $currency )
        {
            $serviceInfo = array( 
                'price' => $price,
                'detail' => $detail,
                'concept' => $concept,
                'currency' => $currency,
            );
        }
        else {
            return json_encode(array('error' => 'Faltante de parametros de servicio de pago'));
        }

		$display = 'url_redirect';
		if ($extraParams['display']) 
		{
			$display = $extraParams['display'];
		}

		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(1,20),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'payment',
			'notification_uri' => $extraParams['notification_uri'],
			'display' => $display,
			'service_info' => $serviceInfo
		);

		$result = $tuidd->consumeService( $service);

		return $result;
	}


	/**
	 * Consume un servicio de notificacion. 
	 * 
	 * @param  string $service_id [ID de servicio]
	 * @param  string $title [titulo de la publicidad]
	 * @param  string $message [mensaje de la publicidad]
	 * @param  string $link [link de la publicidad]
	 * @param  [type] $authInfo [description]
	 * @param  [type] $extraParams [description]
	 * @return array
	 *  int $status Estado de envío de publicidad
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public static function sendNotification ( $service_id, $title=NULL, $message=NULL, $link=NULL, $authInfo, $extraParams = NULL)
	{
		$tuidd = TuiddClient::getSingleton();
        $tuidd->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();

        if ( $service_id ) {
            $serviceInfo = array( 
                'service_id' => $service_id
            );
        }
        else {
            return json_encode(array('error' => 'Service ID vacio'));
        }

        if ( $title )
        {
            $serviceInfo['title'] = $title;
        }
        if ( $message )
        {
            $serviceInfo['message'] = $message;
        }
        if ( $link )
        {
            $serviceInfo['link'] = $link;
        }
        

		$display = 'url_redirect';
		if ($extraParams['display']) 
		{
			$display = $extraParams['display'];
		}

		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(1,20),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'notification',
			'notification_uri' => $extraParams['notification_uri'],
			'display' => $display,
			'service_info' => $serviceInfo
		);

		$result = $tuidd->consumeService( $service );

		return $result;
	}


	/**
	 * Consume un servicio de encuesta mediante el id del servicio. 
	 * 
	 * @param  [type] $serviceID [titulo de la publicidad]
	 * @param  [type] $authInfo [description]
	 * @param  [type] $extraParams [description]
	 * @return array
	 *  int $status Estado de envío de publicidad
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public function sendSurvey ( $service_id, $authInfo, $extraParams = NULL)
	{
		$tuidd = TuiddClient::getSingleton();
        $tuidd->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();

		$display = 'url_redirect';
		if ($extraParams['display'])
		{
			$display = $extraParams['display'];
		}

		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(0,1000),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'survey',
			'display' => $display,
			'service_info' => array(
					'service_id' => $service_id
				)
		);

		$result = $tuidd->consumeService( $service );

		return $result;
	}


	/**
	 * Consume un servicio de cupon mediante la informacion enviada por parametro. 
	 * 
	 * @param  string $service_id [ID de servicio]
	 * @param  string $title [titulo del cupon]
	 * @param  string $message [mensaje del cupon]
	 * @param  string $coupon_type [tipo de cupon ]
	 * @param  string $link [link del cupon]
	 * @param  string $image_link [link de la imagen de cupon]
	 * @param  array $authInfo [description]
	 * @param  array [description]
	 * @return array
	 *  int $status Estado de envío de publicidad
	 *      200: OK
	 *      400: BAD REQUEST 
	 *      401: UNAUTHORIZED
     *      500: INTERNAL SERVER ERROR
	 *  int $transaction_id ID de transacción de servicio
	 */
	public static function sendCoupon ( $service_id, $title=NULL, $message=NULL, $coupon_type=NULL, $link=NULL, $image_link=NULL, $authInfo, $extraParams = NULL )
	{
		$tuidd = TuiddClient::getSingleton();
        $tuidd->generateStateAndNonce();
        $tuidd = TuiddClient::getSingleton();
        $state = $tuidd->getState();

        if ( $service_id ) {
            $serviceInfo = array( 
                'service_id' => $service_id
            );
        }
        else {
            return json_encode(array('error' => 'Service ID vacio'));
        }

        if ( $title )
        {
            $serviceInfo['title'] = $title;
        }
        if ( $message )
        {
            $serviceInfo['message'] = $message;
        }
        if ( $link )
        {
            $serviceInfo['link'] = $link;
        }
        if ( $image_link )
        {
            $serviceInfo['image_link'] = $image_link;	
        }
        if ( $coupon_type )
        {
            $serviceInfo['coupon_type'] = $coupon_type;
        }

		$display = 'url_redirect';
		if ($extraParams['display']) 
		{
			$display = $extraParams['display'];
		}

		$service = array(
			'service_token' => $authInfo['service_token'],
			'seq_num' => mt_rand(1,20),
			'state' => $state,
			'login_hint' => $authInfo['login_hint'],
			'service_type' => 'coupon',
			'display' => $display,
			'service_info' => $serviceInfo
		);
		
		$result = $tuidd->consumeService( $service );

		return $result;
	}
}

?>
